@@ -305,10 +305,10 @@ class FriendlyDB(log.Logger):
             [self.tables.__setitem__(t[0],[]) for t in self.tuples2lists(q.fetchall())]
         return sorted(self.tables.keys())
    
-    def getTableCols(self,table):
+    def getTableCols(self,table,load=False):
         ''' Returns the column names for the given table, and stores these values in the tables dict. '''
         self.getTables()
-        if not self.tables[table]:
+        if load or not self.tables[table]:
             q=self.Query('describe %s'%table,False)
             [self.tables[table].append(t[0]) for t in self.tuples2lists(q.fetchall())]
         return self.tables[table]
