@@ -1,4 +1,5 @@
-#!/usr/bin/env python2.5
+#!/usr/bin/env python2
+
 """ @if gnuheader
 #############################################################################
 ##
@@ -56,24 +57,25 @@ from fandango.dicts import CaselessDefaultDict,CaselessDict
 from fandango.arrays import TimedQueue
 from fandango.dynamic import DynamicDS,USE_STATIC_METHODS
 
+#EventThread.DEFAULT_PERIOD_ms = 1000 #0.1
 
 ########################################################################################
 ## Device servers template
 
-class Dev4Tango(PyTango.Device_4Impl,log.Logger):
+class Dev4Tango(PyTango.LatestDeviceImpl,log.Logger):
     """
     See documentation at doc/devices/Dev4Tango.rst
     
     This class provides several new features to TangoDevice implementations.
     By including log.Logger it also includes objects.Object as parent class.
     It allows to use call__init__(self, klass, *args, **kw) to avoid multiple inheritance from same parent problems.
-    Therefore, use self.call__init__(PyTango.Device_4Impl,cl,name) instead of PyTango.Device_4Impl.__init__(self,cl,name)
+    Therefore, use self.call__init__(PyTango.LatestDeviceImpl,cl,name) instead of PyTango.LatestDeviceImpl.__init__(self,cl,name)
     
-    It also allows to connect several devices within the same server or not usint taurus.core
+    It also allows to connect several devices within the same server or not
     """
     
     def __init__(self,cl,name):
-        self.call__init__(PyTango.Device_4Impl,cl,name)
+        self.call__init__(PyTango.LatestDeviceImpl,cl,name)
         self.init_logger()
 
     def trace(self,prio,s):
@@ -92,7 +94,7 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
     def set_state(self,state):
         self._state = state
         #type(self).mro()[type(self).mro().index(Dev4Tango)+1].set_state(self,state)
-        PyTango.Device_4Impl.set_state(self,state)
+        PyTango.LatestDeviceImpl.set_state(self,state)
         
     def get_state(self):
         #@Tango6
@@ -180,7 +182,7 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
     ##@name Device management methods
     #@{
     
-    def init_logger(self,name = None, use_tango = False):
+    def init_logger(self,name=None, use_tango=False, use_print=False, level="INFO"):
         """ 
         A default initialization for the Logger class 
         
@@ -192,11 +194,12 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
             #Check if this class inherits from Logger
             if isinstance(self,log.Logger): 
                 self.call__init__(log.Logger,name,use_tango=use_tango,
+                    use_print=use_print,level=level,
                     format='%(levelname)-8s %(asctime)s %(name)s: %(message)s')
                 if hasattr(self,'LogLevel'): self.setLogLevel(self.LogLevel)
                 self.info('Logger streams initialized (error,warning,info,debug)')
             else:
-                #try to use Tango Streams
+                print("Using Tango Loggging Streams")
                 try:
                   self.error,self.warning,self.info,self.debug = self.error_stream,self.warn_stream,self.info_stream,self.debug_stream
                 except:
@@ -225,7 +228,7 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
     def get_device_properties(self,myclass):
         self.info('In Dev4Tango.get_device_properties(%s) ...' % str(myclass))
 
-        PyTango.Device_4Impl.get_device_properties(self,myclass)
+        PyTango.LatestDeviceImpl.get_device_properties(self,myclass)
         if hasattr(self,'LogLevel'):
           self.setLogLevel(self.LogLevel)
 
@@ -301,32 +304,48 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
     ##@name DevChild like methods
     #@{    
     
-    def subscribe_external_attributes(self,device,attributes,use_tau=False):
+    def subscribe_external_attributes(self,device,attributes,**kw):
         neighbours = self.get_devs_in_server()
-        self.info('In subscribe_external_attributes(%s,%s): Devices in the same server are: %s'%(device,attributes,neighbours.keys()))
-        if not hasattr(self,'ExternalAttributes'): self.ExternalAttributes = CaselessDict()
-        if not hasattr(self,'PollingCycle'): self.PollingCycle = 3000
-        if not hasattr(self,'last_event_received'): self.last_event_received = 0
-        if not hasattr(self,'_state'): self._state = PyTango.DevState.INIT
-        if not hasattr(self,'events_error'): self.events_error = ''
-        if not hasattr(self,'last_update'): self.last_update = 0
+        self.info(('In subscribe_external_attributes(%s,%s): '
+            'Devices in the same server are: %s') % 
+            (device,attributes,neighbours.keys()))
+        if not hasattr(self,'ExternalAttributes'): 
+            self.ExternalAttributes = CaselessDict()
+        if not hasattr(self,'PollingCycle'): 
+            self.PollingCycle = 3000
+        if not hasattr(self,'last_event_received'): 
+            self.last_event_received = 0
+        if not hasattr(self,'_state'): 
+            self._state = PyTango.DevState.INIT
+        if not hasattr(self,'events_error'): 
+            self.events_error = ''
+        if not hasattr(self,'last_update'): 
+            self.last_update = 0
+            
         device = device.lower()
         deviceObj = neighbours.get(device,None)
-        self.use_tau = getattr(self,'use_tau',TAU and use_tau)
-        if self.use_tau and deviceObj is None:
-            for attribute in attributes: #Done in two loops to ensure that all Cache objects are available
-                self.info ('::init_device(%s): Configuring %s.Attribute for %s' % (self.get_name(),str(TAU.__name__),device+'/'+attribute))
+        
+        if deviceObj is None:
+            #Done in two loops to ensure that all Cache objects are available
+            for attribute in attributes: 
+                self.info ('::init_device(%s): Configuring callback for %s'
+                    % (self.get_name(),device+'/'+attribute))
                 aname = (device+'/'+attribute).lower()
-                at = TAU.Attribute(aname)
-                self.debug('Adding Listener to %s ...'%type(at))
+                at = fandango.callbacks.EventSource(aname,log_level='INFO',
+                    keeptime=250,polling_period=self.PollingCycle)
+                #at.setLogLevel('INFO')
+                self.info('Adding Listener to %s(%s)'%(type(at),aname))
                 at.addListener(self.event_received)
                 self.debug('Changing polling period ...')
                 at.changePollingPeriod(self.PollingCycle)
                 self.ExternalAttributes[aname] = at
+                
         else: #Managing attributes from internal devices
-            self.info('===========> Subscribing attributes from an internal or non-tau device')
+            self.info('==> Subscribing to an internal device')
             for attribute in attributes:
-                self.ExternalAttributes[(device+'/'+attribute).lower()] = fakeAttributeValue(attribute,None,parent=deviceObj,device=device)
+                self.ExternalAttributes[(device+'/'+attribute).lower()] = \
+                    fakeAttributeValue(attribute,None,parent=deviceObj,device=device)
+
             import threading
             if not hasattr(self,'Event'): self.Event = threading.Event()
             if not hasattr(self,'UpdateAttributesThread'):
@@ -413,9 +432,14 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
         while not self.Event.isSet():
             waittime = 3.
             try:
-                serverAttributes = [a for a,v in self.ExternalAttributes.items() if isinstance(v,fakeAttributeValue)]
+                serverAttributes = [a for a,v in self.ExternalAttributes.items() 
+                                    if isinstance(v,fakeAttributeValue)]
                 waittime = 1e-3*self.PollingCycle/(len(serverAttributes)+1)
-                for aname in serverAttributes:
+                
+                for ii,aname in enumerate(sorted(serverAttributes)):
+                    self.Event.wait(waittime)
+                    print('ExternalAttributes %d/%d: %s' 
+                          % (ii,len(serverAttributes),aname))
                     self.last_update = time.time()
                     try:
                         if self.Event.isSet():
@@ -423,30 +447,37 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
                             break
                         attr = self.ExternalAttributes[aname]
                         device,attribute = aname.rsplit('/',1)
-                        self.debug('====> updating values from %s(%s.%s) after %s s'%(type(attr.parent),device,attribute,waittime))
+                        self.debug('====> updating values from %s(%s.%s) '
+                            'after %s s'%(type(attr.parent),device,attribute,waittime))
                         event_type = fakeEventType.lookup['Periodic']
+                        
                         try:
                             attr.read(cache=False)
                         except Exception,e:
-                            print '#'*80
+                            print('#'*80)
                             event_type = fakeEventType.lookup['Error']
                             self.events_error = except2str(e) #traceback.format_exc()
-                            print self.events_error
-                            
-                        self.info('Sending fake event: %s/%s = %s(%s)' % (device,attr.name,event_type,attr.value))
+                            print(self.events_error)
+                    
+                        self.info('Sending fake event: %s/%s = %s(%s)' 
+                            % (device,attr.name,event_type,attr.value))
                         self.event_received(device+'/'+attr.name,event_type,attr)
+                        
                     except: 
                         self.events_error = traceback.format_exc()
-                        print 'Exception in %s.update_attributes(%s): \n%s' % (self.get_name(),attr.name,self.events_error)
-                    self.Event.wait(waittime)
+                        print('Exception in %s.update_attributes(%s): \n%s' 
+                            % (self.get_name(),attr.name,self.events_error))
+                            
                     #End of for
+                print('#$-'*30)
                 self.Event.wait(waittime)
             except:
                 self.events_error = traceback.format_exc()
                 self.Event.wait(3.)
-                print self.events_error
-                print 'Exception in %s.update_attributes()' % (self.get_name())
+                print(self.events_error)
+                print('Exception in %s.update_attributes()' % (self.get_name()))
             #End of while
+
         self.info('%s.update_attributes finished')
         if not self.events_error.replace('None','').strip():
             self.events_error = traceback.format_exc()
@@ -458,9 +489,15 @@ class Dev4Tango(PyTango.Device_4Impl,log.Logger):
         This method must be overriden in child classes.
         """
         #self.info,debug,error,warning should not be used here to avoid conflicts with taurus.core logging
-        def log(prio,s): print '%s %s %s: %s' % (prio.upper(),time.strftime('%Y-%m-%d %H:%M:%S',time.localtime()),self.get_name(),s)
+        def log(prio,s): 
+            print('%s %s %s: %s' % (prio.upper(),
+            time.strftime('%Y-%m-%d %H:%M:%S',
+            time.localtime()),self.get_name(),s))
         self.last_event_received = time.time()
-        log('info','In Dev4Tango.event_received(%s(%s),%s,%s) at %s'%(type(source).__name__,source,fakeEventType[type_],type(attr_value).__name__,self.last_event_received))
+        etype = fakeEventType.get(type_,type_)
+        log('info','In Dev4Tango.event_received(%s(%s),%s,%s) at %s'
+            %(type(source).__name__,source,etype,
+              type(attr_value).__name__,self.last_event_received))
         return
         
     ##@}        
