@@ -206,7 +206,7 @@ class ThreadDict(dict):
             return
         if self.delay:
             self.event.wait(self.delay)
-        print 'In ThreadDict.start(), keys are: %s' % self.threadkeys()   
+        print('In ThreadDict.start()')#, keys are: %s' % self.threadkeys()   
         self.event.clear()
         self._Thread = threading.Thread(target=self.run)
         self._Thread.setDaemon(True)
@@ -235,7 +235,7 @@ class ThreadDict(dict):
         self.stop()
         
     def run(self):
-        self.tracer('In ThreadDict.run()')
+        self.tracer('In ThreadDict.run(%s)' % str(self.threadkeys()))
         self.event.wait(self.get_timewait())
         while not self.event.isSet():
             self.set_last_cycle_start(time.time())
@@ -243,10 +243,11 @@ class ThreadDict(dict):
             if self._last_read and self._last_read!=keys[-1]: #Thread stopped before finishing the cycle!
                 keys = keys[keys.index(self._last_read)+1:]
             for k in keys:
-                if self.trace: self.tracer('ThreadDict::run(): updating %s'%str(k))
                 try:
                     if (time.time()-self._updates.get(k,0))>self._periods.get(k,0):
                         #if period=0 or if not updated the condition applies
+                        if self.trace: 
+                            self.tracer('ThreadDict::run(): updating %s'%str(k))                        
                         self.__getitem__(k,hw=True)
                 except Exception,e:
                     if self.trace:
@@ -1027,6 +1028,12 @@ class Enumeration:
         self._uniqueId += 1
         return n
     
+    def get(self,k,default=None):
+        return self.__getitem__(k) if k in self else default
+    
+    def __contains__(self,i):
+        return i in self.reverseLookup or i in self.lookup
+    
     def __getitem__(self, i):
         if type(i) == types.IntType:
             return self.whatis(i)
