@@ -87,8 +87,15 @@ def first(seq,default=Exception):
             #raise e #if .next() also doesn't work throw unsubscriptable exception
     return
 
-def last(seq,MAX=1000,default=Exception):
-    """Returns last element of sequence"""
+def last(seq,MAX=50000,default=Exception):
+    """
+    Returns last element of sequence
+    
+    default value controls what happens if end is not reached
+    default = exception, raises error if seq is a generator longer than MAX
+    default = None, returns last value read, or None with empty list
+    else, will return default if MAX is reached
+    """
     try:
         return seq[-1]
     except Exception,e:
@@ -98,12 +105,17 @@ def last(seq,MAX=1000,default=Exception):
             if default is not Exception:
                 return default
             else:
-                raise e #if .next() also doesn't work throw unsubscriptable exception
+                raise e
         try:
-            for i in range(1,MAX):
-                n = seq.next()
+            for i in range(1,MAX+1):
+                n = seq.next() #will except when reaching the end
             if i>(MAX-1):
-                raise IndexError('len(seq)>%d'%MAX)
+                if default is Exception:
+                    raise IndexError('MAX index reached')
+                elif default is None:
+                    return n
+                else:
+                    return default
         except StopIteration,e: #It catches generators end
             return n
     return
@@ -141,10 +153,17 @@ def floor(x,unit=1):
     return unit*int(x/unit)
     
 def xor(A,B):
-    """Returns (A and not B) or (not A and B);
-    the difference with A^B is that it works also with different types and returns one of the two objects..
     """
-    return (A and not B) or (not A and B)
+    Returns (A and not B) or (not A and B);
+    the difference with A^B is that it works also with different types 
+    and returns one of the two objects..
+    If both objects are True, False is returned
+    """
+    v = (A and not B) or (not A and B)
+    if v is True and (A and not B):
+            return A
+    else:
+        return v
 
 def reldiff(x,y,floor=None): 
     """
@@ -197,7 +216,8 @@ def join(*seqs):
 def djoin(a,b):
     """ This method merges dictionaries and/or lists """
     if not any(map(isDictionary,(a,b))): return join(a,b)
-    other,dct = sorted((a,b),key=isDictionary) 
+    other, dct = sorted((a,b),key=isDictionary)
+    dct = dict(dct)
     if not isDictionary(other): 
         other = dict.fromkeys(other if isSequence(other) else [other,])
     for k,v in other.items():
@@ -205,12 +225,21 @@ def djoin(a,b):
     return dct
   
 def kmap(method,keys,values=None,sort=True):
+    """
+    Given a method and a list of keys, this method will return a list
+    of (key,method(key)) values.
+    
+    @param values: list of values to pass as argument instead of keys
+    @param sort: return values sorted by key
+    
+    """
     g = ((k,method(k if not values else values[i])) 
            for i,k in enumerate(keys))
     return sorted(g) if sort else list(g)
-__test__['kmap'] = [
-  {'args':[str.lower,'BCA','YZX',False],'result':[('A', 'x'), ('B', 'y'), ('C', 'z')]}
-]
+
+#__test__['kmap'] = [
+  #{'args':[lowstr,'BCA','YZX',False],'result':[('A', 'x'), ('B', 'y'), ('C', 'z')]}
+#]
         
 def splitList(seq,split):
     """splits a list in lists of 'split' size"""
@@ -218,15 +247,25 @@ def splitList(seq,split):
     return [seq[i:i+split] for i in range(len(seq))[::split]]
     
 def contains(a,b,regexp=True):
-    """ Returns a in b; using a as regular expression if wanted """
-    return inCl(a,b,regexp)
+    """ 
+    Returns a in b; 
+    using a as regular expression if wanted 
+    """
+    if a in b:
+        return True
+    elif regexp:
+        return inCl(a,b,regexp)
 
 def anyone(seq,method=bool):
-    """Returns first that is true or last that is false"""
-    if not seq: return False
+    """
+    Like any(), but returns first that is true or last that is false
+    """
+    if not seq: 
+        return False
     s = None
     for s in seq:
-        if method(s): return s
+        if method(s): 
+            return s
     return s if not s else None
 
 def everyone(seq,method=bool):
@@ -243,30 +282,29 @@ def setitem(mapping,key,value):
 
 def getitem(mapping,key):
     return mapping[key]
-  
-def setlocal(key,value):
-    setitem(locals(),key,value)
 
 ########################################################################
 ## Regular expressions 
 ########################################################################
         
 def matchAll(exprs,seq):
-    """ Returns a list of matched strings from sequence.
-    If sequence is list it returns exp as a list.
+    """ 
+    Returns strings in seq that are matched by all expression in exprs
     """
     exprs,seq = toSequence(exprs),toSequence(seq)
     if anyone(isRegexp(e) for e in exprs):
-        exprs = [(e.endswith('$') and e or (e+'$')) for e in exprs]
-        return [s for s in seq if anyone(matchCl(e,s) for e in exprs)]
+        return [s for s in seq if all(matchCl(e,s,terminate=True) for e in exprs)]
     else:
-        return [s for s in seq if s in exprs]
+        exprs = [lowstr(e) for e in exprs]
+        return [s for s in seq if any(e in lowstr(s) for e in exprs)] #lowstr(s)
     
 def matchAny(exprs,seq):
-    """ Returns seq if any of the expressions in exp is matched, if not it returns None """
+    """ 
+    Returns seq if any of the expressions in exp is matched, if not it returns None
+    """
     exprs = toSequence(exprs)
     for exp in exprs:
-        if matchCl(exp,seq): 
+        if matchCl(exp,seq,terminate=True): 
             #print '===============>   matchAny(): %s matched by %s' % (seq,exp)
             return seq
     return None
@@ -299,15 +337,20 @@ def matchTuples(mapping,key,value):
     return True
         
 def inCl(exp,seq,regexp=True):
-    """ Returns a caseless "in" boolean function, using regex if wanted """
+    """
+    it returns True if exp is contained by seq
+    seq may be an string or a list of strings
+    all comparisons are done caseless
+    """
     if isString(seq):
-        return searchCl(exp,seq) if (regexp and isRegexp(exp)) else exp.lower() in seq.lower()
-    if seq is not None and len(seq):
+        return (searchCl(exp,seq) if (regexp and isRegexp(exp)) 
+            else lowstr(exp) in lowstr(seq))
+    elif seq is not None and len(seq):
         if not isSequence(seq):
             seq = toList(seq)
         for s in seq:
             s = str(s)
-            m = matchCl(exp,s) if regexp else exp.lower().strip()==s.lower().strip()
+            m = matchCl(exp,s) if regexp else lowstr(exp)==lowstr(s)
             if m: 
                 return m
     else:
@@ -316,14 +359,14 @@ def inCl(exp,seq,regexp=True):
 def matchCl(exp,seq,terminate=False,extend=False):
     """ Returns a caseless match between expression and given string """
     try:
-        exp,seq = str(exp),str(seq)
+        exp,seq = str(exp),lowstr(seq)
         if extend:
             if '&' in exp:
                 return all(matchCl(e.strip(),seq,terminate=False,extend=True) 
                         for e in exp.split('&'))
             if re.match('^[!~]',exp):
                 return not matchCl(exp[1:],seq,terminate,extend=True) 
-        return re.match(toRegexp(exp,terminate=terminate,lower=True),seq.lower())
+        return re.match(toRegexp(exp.lower(),terminate=terminate),seq)
     except:
         #print('matchCl(%s,%s,%s,%s) failed'%(exp,seq,terminate,extend))
         traceback.print_exc()
@@ -335,14 +378,14 @@ def searchCl(exp,seq,terminate=False,extend=False):
     """ Returns a caseless regular expression search between 
     expression and given string """
     try:
-        exp,seq = str(exp),str(seq)
+        exp,seq = str(exp),lowstr(seq)
         if extend:
             if '&' in exp:
                 return all(searchCl(e.strip(),seq,terminate=False,extend=True) 
                         for e in exp.split('&'))
             if re.match('^[!~]',exp):
                 return not searchCl(exp[1:],seq,terminate,extend=True)
-        return re.search(toRegexp(exp.lower(),terminate=terminate),seq.lower())
+        return re.search(toRegexp(exp.lower(),terminate=terminate),seq)
     except:
         #print('searchCl(%s,%s,%s,%s) failed'%(exp,seq,terminate,extend))
         traceback.print_exc()
@@ -449,99 +492,71 @@ def filtersmart(seq,filters):
         else: raw.append(parse(f))
     return [s for s in seq if (not any(matchCl(n,s) for n in neg)) and any(matchCl(r,s) for r in raw) and (not comp or all(matchCl(c,s) for c in comp))]
 
-########################################################################
-## Methods for piped iterators
-## Inspired by Maxim Krikun [ http://code.activestate.com/recipes/276960-shell-like-data-processing/?in=user-1085177]
-########################################################################
-        
-class Piped:
-    """This class gives a "Pipeable" interface to a python method:
-        cat | Piped(method,args) | Piped(list)
-        list(method(args,cat))
-    e.g.: 
-    class grep:
-        #keep only lines that match the regexp
-        def __init__(self,pat,flags=0):
-            self.fun = re.compile(pat,flags).match
-        def __ror__(self,input):
-            return ifilter(self.fun,input) #imap,izip,count,ifilter could ub useful
-    cat('filename') | grep('myname') | printlines
-    """
-    import itertools
-    def __init__(self,method,*args,**kwargs):
-        self.process=partial(method,*args,**kwargs)
-    def __ror__(self,input):
-        return imap(self.process,input)
-        
-class iPiped:
-    """ Used to pipe methods that already return iterators 
-    e.g.: hdb.keys() | iPiped(filter,partial(fandango.inCl,'elotech')) | plist
-    """
-    def __init__(self,method,*args,**kwargs): self.process = partial(method,*args,**kwargs)
-    def __ror__(self,input): return self.process(input)
-    
-class zPiped:
-    """ 
-    Returns a callable that applies elements of a list of tuples to a set of functions 
-    e.g. [(1,2),(3,0)] | zPiped(str,bool) | plist => [('1',True),('3',False)]
-    """
-    def __init__(self,*args): self.processes = args
-    def __ror__(self,input): return (tuple(p(i[j]) for j,p in enumerate(self.processes))+tuple(i[len(self.processes):]) for i in input)
-    
-pgrep = lambda exp: iPiped(lambda input: (x for x in input if inCl(exp,x)))
-pmatch = lambda exp: iPiped(lambda input: (x for x in input if matchCl(exp,str(x))))
-pfilter = lambda meth=bool,*args: iPiped(filter,partial(meth,*args))
-ppass = Piped(lambda x:x)
-plist = iPiped(list)
-psorted = iPiped(sorted)
-pdict = iPiped(dict)
-ptuple = iPiped(tuple)
-pindex = lambda i: Piped(lambda x:x[i])
-pslice = lambda i,j: Piped(lambda x:x[i,j])
-penum = iPiped(lambda input: izip(count(),input) )
-pzip = iPiped(lambda i:izip(*i))
-ptext = iPiped(lambda input: '\n'.join(imap(str,input)))
 
 ##############################################################################
 
+FalseStrings = ('no','false','','0','none')
+NoneStrings = ('none','null','nan','')
+TrueStrings = ('true','yes')
+NaN = float('nan')
+inf = float('inf')
+
 def fbool(x):
     """
-    Returns all(x) if sequence else bool(x) or False
+    Instead of just considering sequences as True, it evaluates contents
+    Returns all(x) if sequence and not empty else bool(x)
     """
     if isSequence(x):
+        if not len(x):
+            return False
         return all(x)
     else:
         return bool(x)
     
 def notNone(arg,default=None):
-    """ Returns arg if not None, else returns default. """
+    """ 
+    Returns arg if not None, else returns default. 
+    """
     return [arg,default][arg is None]
 
 def isTrue(arg):
-    """ Returns True if arg is not None, not False and not an empty iterable. """
-    if hasattr(arg,'__len__'): return len(arg)
-    else: return arg
+    """ 
+    This method allows to evaluate strings and numpy arrays as booleans
+    """
+    if type(arg) not in (int,bool,float):
+        if isSequence(arg) or isMapping(arg):
+            arg = len(arg)
+        elif isString(arg):
+            arg = lowstr(arg) not in FalseStrings
+    return bool(arg)
+       
+def isNaN(arg):
+    return (isinstance(arg,float) and math.isnan(arg) or 
+                (isString(arg) and lowstr(arg) == 'nan'))
     
-NaN = float('nan')
-inf = float('inf')
-    
-def isNaN(seq):
-    return (isinstance(seq,(int,float)) and math.isnan(seq) or 
-                (isString(seq) and seq.lower().strip() == 'nan'))
-    
-def isNone(seq):
-    return seq is None or (isString(seq) and seq.lower().strip() in ('none','null','nan',''))
+def isNone(arg):
+    return arg is None or (isString(arg) and lowstr(arg) in NoneStrings)
 
-def isFalse(seq):
-    return not seq or str(seq).lower().strip() in ('false','0','no')
+def isFalse(arg):
+    # considers strings as boolean representations
+    # 0/1 and none/nan are not considered as booleans
+    if isSequence(arg) or isMapping(arg):
+        arg = len(arg)    
+    return not arg or lowstr(arg) in FalseStrings
 
-def isBool(seq,is_zero=True):
+def isBool(arg,is_zero=True):
+    """
+    Input may be an string, but not an integer
+    True/yes/false/no are considered valid boolean strings
+    """
     codes = ['true','yes','false','no']
-    if is_zero: codes+=['0','1']
-    if seq in (True,False):
+    if is_zero: 
+        codes+=['0','1']
+    if arg in (True,False):
         return True
-    elif isString(seq):
-        return seq.lower() in codes #none/nan will not be considered boolean
+    elif isString(arg):
+        #none/nan will not be considered boolean
+        return lowstr(arg) in codes 
     else:
         return False
     
@@ -582,9 +597,9 @@ def isNumber(seq):
         return True
     except: return False
 
-def isDate(seq):
+def isDate(seq,cad=''):
     try:
-        return str2time(seq)
+        return seq and str2time(seq,cad=cad)
     except:
         return False
 
@@ -593,15 +608,22 @@ def isGenerator(seq):
     # A generator check must be added to the rest of methods in this module!
     return isinstance(seq,GeneratorType)
     
-def isSequence(seq,INCLUDE_GENERATORS = True):
-    """ It excludes Strings, dictionaries but includes generators"""
+def isSequence(seq,iterators= True,**kwargs):
+    """ 
+    It excludes Strings, dictionaries but includes generators
+    unless include_generators=False is set as argument
+    
+    if include_generators is False and iterators is False only fixed-length objects are accepted
+    """
+    if kwargs.get('INCLUDE_GENERATORS',False):
+        iterators = True
     if any(isinstance(seq,t) for t in (list,set,tuple)): 
         return True
     if isString(seq): 
         return False
     if hasattr(seq,'items'): 
         return False
-    if INCLUDE_GENERATORS:
+    if iterators:
         if hasattr(seq,'__iter__'): 
             return True
     elif hasattr(seq,'__len__'): 
@@ -676,7 +698,7 @@ def str2float(seq):
 
 def str2bool(seq):
     """ It parses true/yes/no/false/1/0 as booleans """
-    return seq.lower().strip() not in ('false','0','none','no')
+    return lowstr(seq) not in ('false','0','none','no')
   
 def str2bytes(seq):
     """ Converts an string to a list of integers """
@@ -834,6 +856,11 @@ def code2atoms(code):
     l1 = filter(bool,map(str.strip,l1))
     #l2 = [a for l in l1 for a in str2list(l,ops,1,-1)]
     return l1
+
+def lowstr(s,strip=True):
+    """ converts anything to str(s).lower().strip() """
+    s = str(s).lower()
+    return s.strip() if strip else s
     
 def shortstr(s,max_len=144,replace={'\n':';'}):
     """ Obtain a shorter string """
@@ -1020,6 +1047,11 @@ def tuple2time(tup):
     return time.mktime(tup)
 
 def date2time(date,us=True):
+    """
+    This method would accept both timetuple and timedelta
+    in order to deal with times coming from different
+    api's with a single method
+    """
     try:
       t = tuple2time(date.timetuple())
       us = us and getattr(date,'microsecond',0)
@@ -1224,7 +1256,7 @@ def ifThen(condition,callback,falsables=tuple()):
     else:
         return condition
     
-def call(args=None,kwargs=None,locals_=None):
+def call(args=None,kwargs=None,locals_=None, debug=False):
     """
     Calls a method from local scope parsing a pipe-like argument list
     
@@ -1237,6 +1269,7 @@ def call(args=None,kwargs=None,locals_=None):
         args = sys.argv[1:]
         
     kwargs, export = kwargs or {}, ''
+    #args = map(str.strip,args)
     f,args = args[0],args[1:]
     
     if not isCallable(f):
@@ -1249,9 +1282,12 @@ def call(args=None,kwargs=None,locals_=None):
                 return '%s:\n%s' % (n,o.__doc__)
             else:
                 m = [k for k,v in locals_.items() if isCallable(v)]
-                return ('\n'.join(sorted(m,key=str.lower)))
+                return ('\n'.join(sorted(m,key=lowstr)))
         f = locals_.get(f,None) 
         
+    if debug:
+        print('%s(%s)' % (f,args))
+        
     if all(isString(a) for a in args):
         kwargs = [a for a in args if '=' in a]
         args = [a for a in args if a not in kwargs]
@@ -1281,17 +1317,18 @@ def retry(callable,retries=3,pause=0,args=[],kwargs={}):
   
 def retried(retries=3,pause=0):
   """
-  
+  @decorator
+  Returns a function decorator to execute any method in retried mode
   """
   def retrier(f):
     def retried_f(*args,**kwargs):
       return retry(f,retries=retries,pause=pause,args=args,kwargs=kwargs)
     return retried_f
   return retrier
-    
+
 def evalF(formula):
     """
-    Returns a function that executes the formula passes as argument.
+    Returns a compiled function that executes the formula passed as argument.
     The formula should use x,y,z as predefined arguments, or use args[..] array instead
     
     e.g.:
@@ -1300,8 +1337,11 @@ def evalF(formula):
     It is optimized to be efficient (but still 50% slower than a pure lambda)
     """
     #return (lambda *args: eval(formula,locals={'args':args,'x':args[0],'y':args[1],'z':args[2]}))
-    c = compile(formula,formula,'eval') #returning a lambda that evals a compiled code makes the method 500% faster
-    return (lambda *args: eval(c,{'args':args,'x':args and args[0],'y':len(args)>1 and args[1],'z':len(args)>2 and args[2]}))
+    #returning a lambda that evals a compiled code makes the method 500% faster
+    c = compile(formula,formula,'eval') 
+    return (lambda *args: 
+        eval(c,{'args':args,'x':args and args[0],
+            'y':len(args)>1 and args[1],'z':len(args)>2 and args[2]}))
 
 def testF(f,args=[],t=5.):
     """
