@@ -91,10 +91,18 @@ except:
 ## Inspection methods
 
 def dirModule(module):
+    """
+    instead of dir(module) that returns everything imported in a file
+    this dirModule() method retuns only the methods explicitly declared
+    in the module file
+    """
     return [a for a,v in list(module.__dict__.items()) 
         if getattr(v,'__module__','') == module.__name__]
 
 def findModule(module):
+    """
+    returns the path to a python module
+    """
     import imp
     if '.' not in module:
         return imp.find_module(module)[1]
@@ -108,7 +116,9 @@ def findModule(module):
 find_module = findModule
 
 def loadModule(module,modulename=None):
-    #Loads a python module either from source file or from module
+    """
+    Loads a python module either from source file or from module
+    """
     import imp
     if modulename or '/' in module or '.py' in module:
         if not modulename:
@@ -240,6 +250,8 @@ class Struct(object):
     """
     Metamorphic type to pass/retrieve data objects as object or dictionary
     
+    Creator arguments should be either empty, a dictionary or keyword arguments.
+    
     s = Struct(name='obj1',value=3.0)
     s.setCastMethod(lambda k,v: str2type)
     s.cast('3.0') : 3.0
@@ -452,7 +464,8 @@ def self_locked(func,reentrant=True):
 
 def NewClass(classname,classparent=None,classdict=None):
     """ 
-    Creates a new class on demand:
+    Creates a new class on demand just by merging an existing class with 
+    a dict containing new methods:
      ReleaseNumber = NewClass('ReleaseNumber',tuple,
             {'__repr__':(lambda self:'.'.join(('%02d'%i for i in self)))})
     """
@@ -540,6 +553,9 @@ class Object(object):
     """
     This class solves some problems when an object inherits from multiple 
     classes and some of them inherit from the same 'grandparent' class
+    
+    Objects inheriting from fandango.Object will be able to call any __init__ method
+    from any parent class disregarding of the inheritance order.
     """ 
 
     def __init__(self):
@@ -840,6 +856,9 @@ class Cached(Decorator):
     This decorator will provide a function that caches up to N different
     executions of a method (for different combinations of arguments) for 
     a given period. It is very similar to functools.lru_cache in py3 
+    
+    You can setup an expire date for the cache, so it is recalculated when
+    time period passes.
     """
     __example__ = """
     
