@@ -224,7 +224,7 @@ class QOptionDialog(QDialogWidget):
       QDialogWidget.accept(self)
         
     def getModel(self):
-      return self._model
+      return getattr(self,'_model',None)
         
 class QExceptionMessage(object):
     def __init__(self,*args):
@@ -526,6 +526,7 @@ def getColorsForValue(value,palette = getattr(colors,'QT_DEVICE_STATE_PALETTE',N
     if value is None:
         return Qt.QColor(Qt.Qt.gray),Qt.QColor(Qt.Qt.black)
     elif hasattr(value,'value'): #isinstance(value,PyTango.DeviceAttribute):
+        import PyTango
         if value.type == PyTango.ArgType.DevState:
             bg_brush, fg_brush = colors.QT_DEVICE_STATE_PALETTE.qbrush(value.value)
         elif value.type == PyTango.ArgType.DevBoolean:
@@ -1106,6 +1107,12 @@ def Draggable(QtKlass):
             dropAction = drag.start(Qt.Qt.CopyAction)
             #tracer('Out of Draggable(%s).mouseMoveEvent'%self._qt_klass__)
             
+        def setModel(self,model):
+            self._model = model
+            
+        def getModel(self):
+            return getattr(self,'_model',None)
+            
     return DraggableQtKlass
 
 QDraggable = Draggable(object)
@@ -1784,10 +1791,18 @@ class QEvaluator(Qt.QWidget):
             traceback.print_exc()
         
     def dir_module(self,*args):
+        """
+        This method gets first argument that is an string as filter
+        Then gets the next arguments that are not string as module
+        It applies a filter on dir(module) and returns matching methods
+        It excludes private members
+        """
         f = fandango.first((a for a in args if isString(a)),'*')
-        if '*' not in f and not f.startswith('_'): f = '*%s*'%f
+        if '*' not in f and not f.startswith('_'): 
+            f = '*%s*'%f
         ks = fandango.first((dir(a) for a in args if not isString(a)),self._locals.keys())
-        return sorted((t for t in fandango.matchAll(f,ks) if f.startswith('_') or not t.startswith('__')),key=str.lower)
+        return sorted((t for t in fandango.matchAll(f,ks) 
+                       if f.startswith('_') or not t.startswith('__')),key=str.lower)
     
     def help(self,m=None):
         try:
