@@ -56,8 +56,11 @@ try:
     from PyTango.utils import EventCallBack
 except: pass
 
-if 'Device_4Impl' not in dir(PyTango):
-    PyTango.Device_4Impl = PyTango.Device_3Impl
+if 'LatestDeviceImpl' not in dir(PyTango):
+   try:
+       PyTango.LatestDeviceImpl = PyTango.Device_5Impl
+   except:
+       PyTango.LatestDeviceImpl = PyTango.Device_4Impl
 
 import fandango
 import fandango.objects as objects
@@ -70,6 +73,9 @@ from fandango.linos import get_fqdn
 from fandango.log import Logger,except2str,printf
 from fandango.excepts import exc2str
 
+global TANGO_DEBUG
+TANGO_DEBUG = os.getenv('TANGO_DEBUG')
+
 #taurus imports, here USE_TAU is defined for all fandango
 global TAU,USE_TAU,TAU_LOGGER
 TAU,USE_TAU = None,False
@@ -94,7 +100,8 @@ def loadTaurus():
     return bool(TAU)
 
 global USE_FQDN
-USE_FQDN = str2type(os.getenv('USE_FQDN','None'))
+#Using FQDN hostname to avoid notifd issues
+USE_FQDN = str2type(os.getenv('USE_FQDN','True')) #'None'))
 
 TANGO_STATES = \
   'ON OFF CLOSE OPEN INSERT EXTRACT MOVING STANDBY FAULT INIT RUNNING ALARM '\
@@ -226,6 +233,8 @@ def get_normal_name(model):
         model = model.split(':')[-1].split('/',1)[-1]
     return model.split('#')[0].strip('/')
 
+get_simple_name = get_normal_name
+
 def get_attr_name(model,default='state'):
     """
     gets just the attribute part of a Tango URI
@@ -250,6 +259,8 @@ def get_dev_name(model,full=True,fqdn=None):
         return get_normal_name(model)
     else:
         return get_full_name(model,fqdn=fqdn)
+    
+get_device_name = get_dev_name
 
 @Cached(depth=100,expire=60)
 def get_tango_host(dev_name='',use_db=False, fqdn=None):
@@ -317,6 +328,7 @@ def set_tango_host(tango_host):
     
     global TangoDatabase,TangoDevice,TangoProxies
     TangoDatabase,TangoDevice,TangoProxies = None,None,None
+    TangoProxies = ProxiesDict(use_tau = USE_TAU)
     import fandango.tango
     for k in dir(fandango.tango):
         f = getattr(fandango.tango,k)
@@ -406,13 +418,20 @@ def get_device(dev,use_tau=False,keep=None,proxy=None,trace=False):
                 r = TAU.Device(dev)
             else:
                 global TangoProxies
+                if TangoProxies is None:
+                    TangoProxies = ProxiesDict(use_tau = USE_TAU)
+                    
                 # Key names managed by ProxiesDict
-                if TangoProxies and (dev in TangoProxies or keep):
-                    if proxy and dev not in TangoProxies:
+                cached = dev in TangoProxies
+                if cached or keep:
+                    if proxy: # and dev not in TangoProxies:
                         TangoProxies[dev] = proxy
+                    if not cached and TANGO_DEBUG:
+                        print('>>>> get_device(%s): TangoProxies.new()' % dev)
                     r = TangoProxies[dev]
-                    cached = True
                 else: 
+                    if TANGO_DEBUG:
+                        print('>>>> get_device(%s): creating new proxy' % dev)
                     r = PyTango.DeviceProxy(dev)
                 
         elif isinstance(dev,PyTango.DeviceProxy) \
@@ -477,14 +496,17 @@ class fakeAttributeValue(object):
                  quality=PyTango.AttrQuality.ATTR_VALID,
                  dim_x=1,dim_y=1,parent=None,device='',
                  error=False,keeptime=0):
-        self.name=name
-        self.device=device or (self.name.rsplit('/',1)[0] 
-                               if '/' in self.name else '')
+        if TANGO_DEBUG:
+            print('new fakeAttributeValue(%s)' % name)
+        self.full_name = name
+        self.name = name.rsplit('/',1)[-1]
+        self.device = (device or (name.rsplit('/',1)[0]) if '/' in name 
+                                 else (parent or ''))
         self.set_value(value,dim_x,dim_y)
         self.set_date(time_ or time.time())
         self.write_value = self.wvalue = None
-        self.quality=quality
-        self.parent=parent
+        self.quality = quality
+        self.parent = parent
         self.error = self.err = error
         self.keeptime = keeptime*1e3 if keeptime<10. else keeptime
         self.lastread = 0
@@ -498,16 +520,20 @@ class fakeAttributeValue(object):
     __str__ = __repr__
         
     def get_name(self): return self.name
+    def get_full_name(self): return self.full_name
+    def get_device(self): return self.device
     def get_value(self): return self.value
     def get_date(self): return self.time
     def get_time(self): return self.time.totime()
     def get_quality(self): return self.quality
     
     def read(self,cache=True):
+        if TANGO_DEBUG:
+            print('fakeAttributeValue.read(%s,%s)' % (self.name,self.parent))
         #Method to emulate AttributeProxy returning an AttributeValue
         if not self.parent:
             self.parent = get_device(self.device,use_tau=False,keep=True)
-        if not cache or 0<self.keeptime<(time.time()-self.read()):
+        if not cache or 0<self.keeptime<(time.time()-self.lastread):
             #it's important to pass self as argument so values will be kept
             import fandango.tango.methods as fmt
             return fmt.read_internal_attribute(self.parent,self) 
