@@ -273,6 +273,11 @@ class DynamicAttribute(object):
                 return ''
             else:
                 return None
+        elif float in (type(value),type(other)):
+            try:
+                value = float(value)
+            except:
+                print('DynamicAttribute.operator(%s): unable to convert to float' % str(value))
 
         result = DynamicAttribute()
         result.quality,result.date,result.primeOlder=self.quality,self.date,self.primeOlder
@@ -311,6 +316,7 @@ class DynamicAttribute(object):
             result.date = min([self.date,other.date]) if self.primeOlder else max([self.date,other.date])
             result.value = method(value,other.value)
         else:
+            #### THIS IS THE DEFAULT FOR MOST OPERATIONS
             #print('%s,%s(%s),%s(%s)' % (method,type(value),value,type(other),other))
             result.value = method(value,other)
         if op_name in ['__nonzero__','__int__','__float__','__long__','__complex__','__index__','__len__','__str__',
@@ -325,6 +331,8 @@ class DynamicAttribute(object):
     def __sub__(  self, other): return self.operator(inspect.currentframe().f_code.co_name,other)
     def __mod__(  self, other):    return self.operator(inspect.currentframe().f_code.co_name,other)
     def __div__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
+    def __truediv__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
+    def __floordiv__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
     def __rshift__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
     def __lshift__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
     #def __truediv__(self,other): return self.value.__class__.__dict__[inspect.currentframe().f_code.co_name](self.value,other)
@@ -334,6 +342,8 @@ class DynamicAttribute(object):
     def __rsub__(  self, other): return self.operator(inspect.currentframe().f_code.co_name,other)
     def __rmod__(  self, other):    return self.operator(inspect.currentframe().f_code.co_name,other)
     def __rdiv__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
+    def __rtruediv__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
+    def __rfloordiv__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
     def __rrshift__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
     def __rlshift__(self,other): return self.operator(inspect.currentframe().f_code.co_name,other)
     #def __rtruediv__(self,other): return self.value.__class__.__dict__[inspect.currentframe().f_code.co_name](self.value,other)
