@@ -62,7 +62,7 @@ def add_new_device(server,klass,device):
           'MyServer/test','MyDevice','my/own/device')
     """
     for c in (server+klass+device):
-      if re.match('[^a-zA-Z0-9\-\/_\+\.]',c):
+      if re.match('[^a-zA-Z0-9\-\/_\+\.\:]',c):
         raise Exception,"CharacterNotAllowed('%s')"%c
     assert server.count('/')==1
     assert clmatch(alnum,klass)
@@ -440,7 +440,7 @@ def get_matching_device_attribute_labels(device,attribute):
                 for a,l in get_device_labels(t,attribute).items() 
                 if check_device(t))
 
-def get_attribute_info(device,attribute):
+def get_attribute_info(model, attribute=None):
     """
     This method returns attribute info in the attr_list format
     It parses values returned by:
@@ -454,16 +454,25 @@ def get_attribute_info(device,attribute):
                 'unit':"mbar",
                 'format':"%4d",
              }],
-    """
-    if isString(device): device = get_device(device)
-    ai = device.get_attribute_config(attribute)
+    """   
+    if isString(model): 
+        if attribute is None:
+            model,attribute = model.rsplit('/',1)
+        model = get_device(model)
+        
+    ai = model.get_attribute_config(attribute)
     types = [CmdArgType.values.get(ai.data_type),
              ai.data_format,ai.writable]
-    if ai.max_dim_x>1: types.append(ai.max_dim_x)
-    if ai.max_dim_y>0: types.append(ai.max_dim_y)
+    
+    if ai.max_dim_x>1: 
+        types.append(ai.max_dim_x)
+    if ai.max_dim_y>0: 
+        types.append(ai.max_dim_y)
+    
     formats = dict((k,getattr(ai,k)) 
         for k,v in (('label',''),('unit','No unit'),('format',''))
         if getattr(ai,k)!=v)
+
     return [types,formats]
   
 @Cached(depth=100,expire=15.)
@@ -1197,7 +1206,8 @@ def check_device_cached(*args,**kwargs):
     """
     return check_device(*args,**kwargs)
 
-def check_attribute(attr,readable=False,timeout=0,brief=False,trace=False):
+def check_attribute(attr,readable=False,timeout=0,brief=False,trace=False,
+                    expire=0):
     """ checks if attribute is available.
     
     Returns None if attribute does not exist, Exception if unreadable, 
@@ -1205,7 +1215,7 @@ def check_attribute(attr,readable=False,timeout=0,brief=False,trace=False):
     
     :param readable: Whether if it's mandatory that the attribute returns 
             a value or if it must simply exist.
-    :param timeout: Checks if the attribute value have been effectively 
+    :param expire: Checks if the attribute value have been effectively 
             updated (check zombie processes).
     :param brief: return just .value instead of AttrValue object
     """
@@ -1218,11 +1228,14 @@ def check_attribute(attr,readable=False,timeout=0,brief=False,trace=False):
                 for s in DeviceProxy(dev).get_attribute_list()],'AttrNotFound'
           proxy = AttributeProxy(attr)
           
+        if timeout:
+            proxy.get_device_proxy().set_timeout_millis(timeout)
+            
         try: 
             attvalue = proxy.read()
             if readable and attvalue.quality == AttrQuality.ATTR_INVALID:
                 return None
-            elif timeout and attvalue.time.totime()<(time.time()-timeout):
+            elif expire and attvalue.time.totime()<(time.time()-expire):
                 return None
             else:
                 if not brief:
@@ -1346,7 +1359,8 @@ def get_device_help(self,str_format='text'):
       traceback.print_exc()
       raise e
     return docsep.join(docs)
-    
+
+@Cached(depth=1,expire=3600,catched=True)
 def get_internal_devices():
     """ Gets all devices declared in the current Tango server """
     try:
@@ -1375,7 +1389,7 @@ def read_internal_attribute(device,attribute):
     if the device is not internal this method will connect to a PyTango Proxy
     the method will return a fakeAttributeValue object
     """
-    print('read_internal_attribute(%s,%s)'%(device,attribute))
+    #print('read_internal_attribute(%s,%s)'%(device,attribute))
     import fandango.dynamic as dynamic
     
     if isString(device):
@@ -1385,15 +1399,25 @@ def read_internal_attribute(device,attribute):
     
     attr = (attribute if isinstance(attribute,fakeAttributeValue) 
                     else fakeAttributeValue(name=attribute,parent=device))
+    #attr = attribute
     
     isProxy = isinstance(device,DeviceProxy)
     isDyn = hasattr(device,'read_dyn_attr')
     aname = attr.name.lower()
+
     if aname=='state': 
-        if isProxy: attr.set_value(device.state())
-        elif hasattr(device,'last_state'): attr.set_value(device.last_state)
-        else: attr.set_value(device.get_state())
-        print('%s = %s' % (attr.name,attr.value))
+        if isProxy: 
+            # read_attribute(state) does not work and may have leaks
+            #print('fandango.read_internal_attribute(): '
+                    #'calling DeviceProxy(%s).state()'
+                    #%(attr.device))     
+            v = device.state()
+            attr.set_value(v)
+        elif hasattr(device,'last_state'): 
+            attr.set_value(device.last_state)
+        else: 
+            attr.set_value(device.get_state())
+        print('%s.%s = %s' % (attr.device,attr.name,attr.value))
         attr.error = ''
     else: 
         if isProxy:
